Package: ClusterFoldSimilarity
Type: Package
Title: Calculate similarity of clusters from different single cell
        samples using foldchanges
Version: 1.8.0
Authors@R: person("Oscar", "Gonzalez-Velasco",
  email = "oscargvelasco@gmail.com",
  role = c("cre", "aut"),
  comment = c(ORCID = "0000-0002-5054-8635"))
Description: This package calculates a similarity coefficient using the fold changes of shared features (e.g. genes) among clusters of different samples/batches/datasets. The similarity coefficient is calculated using the dot-product (Hadamard product) of every pairwise combination of Fold Changes between a source cluster i of sample/dataset n and all the target clusters j in sample/dataset m
License: Artistic-2.0
Encoding: UTF-8
Imports: methods, igraph, ggplot2, scales, BiocParallel, graphics,
        stats, utils, Matrix, cowplot, dplyr, reshape2, Seurat,
        SeuratObject, SingleCellExperiment, ggdendro
Suggests: knitr, rmarkdown, kableExtra, scRNAseq, BiocStyle
RoxygenNote: 7.2.3
biocViews: SingleCell, Clustering, FeatureExtraction, GraphAndNetwork,
        GeneTarget, RNASeq
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/ClusterFoldSimilarity
git_branch: RELEASE_3_23
git_last_commit: ebc46cd
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:24:44 UTC; biocbuild
Author: Oscar Gonzalez-Velasco [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-5054-8635>)
Maintainer: Oscar Gonzalez-Velasco <oscargvelasco@gmail.com>

# Generated by roxygen2: do not edit by hand

export(clusterFoldSimilarity)
export(findCommunitiesSimmilarity)
export(plotClustersGraph)
export(similarityHeatmap)
import(Seurat)
import(ggplot2)
importFrom(BiocParallel,bplapply)
importFrom(Matrix,Matrix)
importFrom(Matrix,rowMeans)
importFrom(SingleCellExperiment,colLabels)
importFrom(SingleCellExperiment,counts)
importFrom(cowplot,align_plots)
importFrom(cowplot,plot_grid)
importFrom(dplyr,"%>%")
importFrom(dplyr,arrange)
importFrom(dplyr,filter)
importFrom(ggdendro,ggdendrogram)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(igraph,V)
importFrom(igraph,cluster_infomap)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,layout_with_fr)
importFrom(igraph,membership)
importFrom(igraph,modularity)
importFrom(methods,is)
importFrom(reshape2,melt)
importFrom(scales,alpha)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,median)
importFrom(stats,optim)
importFrom(stats,pnorm)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(utils,head)

## ----options, include=FALSE, echo=FALSE---------------------------------------
library(BiocStyle)
knitr::opts_chunk$set(eval=TRUE, warning=FALSE, error=FALSE, message=FALSE)

## ----eval=FALSE---------------------------------------------------------------
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("ClusterFoldSimilarity")

## ----setup--------------------------------------------------------------------
library(ClusterFoldSimilarity)

## ----construct----------------------------------------------------------------
library(Seurat)
library(scRNAseq)
library(dplyr)
# Human pancreatic single cell data 1
pancreasMuraro <- scRNAseq::MuraroPancreasData(ensembl=FALSE)
pancreasMuraro <- pancreasMuraro[,rownames(colData(pancreasMuraro)[!is.na(colData(pancreasMuraro)$label),])]
colData(pancreasMuraro)$cell.type <- colData(pancreasMuraro)$label
rownames(pancreasMuraro) <- make.names(unlist(lapply(strsplit(rownames(pancreasMuraro), split="__"), function(x)x[[1]])), unique = TRUE)
singlecell1Seurat <- CreateSeuratObject(counts=counts(pancreasMuraro), meta.data=as.data.frame(colData(pancreasMuraro)))

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(
  table(colData(pancreasMuraro)$label),
  caption = 'Cell-types on pancreas dataset from Muraro et al.', align = "c"
)

## -----------------------------------------------------------------------------
# Human pancreatic single cell data 2
pancreasBaron <- scRNAseq::BaronPancreasData(which="human", ensembl=FALSE)
colData(pancreasBaron)$cell.type <- colData(pancreasBaron)$label
rownames(pancreasBaron) <- make.names(rownames(pancreasBaron), unique = TRUE)

singlecell2Seurat <- CreateSeuratObject(counts=counts(pancreasBaron), meta.data=as.data.frame(colData(pancreasBaron)))

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(
  table(colData(pancreasBaron)$label),
  caption = 'Cell-types on pancreas dataset from Baron et al.', align = "c"
)

## ----results='hide'-----------------------------------------------------------
# Create a list with the unprocessed single-cell datasets
singlecellObjectList <- list(singlecell1Seurat, singlecell2Seurat)
# Apply the same processing to each dataset and return a list of single-cell analysis
singlecellObjectList <- lapply(X=singlecellObjectList, FUN=function(scObject){
scObject <- NormalizeData(scObject)
scObject <- FindVariableFeatures(scObject, selection.method="vst", nfeatures=2000)
scObject <- ScaleData(scObject, features=VariableFeatures(scObject))
scObject <- RunPCA(scObject, features=VariableFeatures(object=scObject))
scObject <- FindNeighbors(scObject, dims=seq(16))
scObject <- FindClusters(scObject, resolution=0.4)
})

## ----fig.wide = TRUE----------------------------------------------------------
# Assign cell-type annotated from the original study to the cell labels:
Idents(singlecellObjectList[[1]]) <- factor(singlecellObjectList[[1]][[]][, "cell.type"])

library(ClusterFoldSimilarity)
similarityTable <- clusterFoldSimilarity(scList=singlecellObjectList, 
                                         sampleNames=c("human", "humanNA"),
                                         topN=1, 
                                         nSubsampling=24)


## ----echo=FALSE---------------------------------------------------------------
knitr::kable(
  head(similarityTable), caption = 'A table of the first 10 rows of the similarity results.', align = "c", 
  format = "html", table.attr = "style='width:70%;'") %>% kableExtra::kable_styling(font_size=10)

## -----------------------------------------------------------------------------
typeCount <- singlecellObjectList[[2]][[]] %>% 
  group_by(seurat_clusters) %>% 
  count(cell.type) %>%
  arrange(desc(n), .by_group = TRUE) %>% 
  filter(row_number()==1)

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(
  cbind.data.frame(typeCount, 
                 matched.type=rep(table(typeCount$seurat_clusters), x=similarityTable[similarityTable$datasetL == "humanNA",]$clusterR)),
  caption = 'Cell-type label matching on pancreas single-cell data.', align = "c"
)

## ----fig.wide = TRUE----------------------------------------------------------
cellCommunities <- findCommunitiesSimmilarity(similarityTable = similarityTable)

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(
  head(cellCommunities)
)

## ----fig.wide = TRUE----------------------------------------------------------
# Retrieve the top 3 similar cluster for each of the clusters:
similarityTable3Top <- clusterFoldSimilarity(scList=singlecellObjectList, 
                                             topN=3,
                                             sampleNames=c("human", "humanNA"), 
                                             nSubsampling=24)

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(
  head(similarityTable3Top),
  caption = 'Similarity results showing the top 3 most similar clusters.', align = "c", 
  format = "html", table.attr = "style='width:70%;'") %>% kableExtra::kable_styling(font_size=10)

## ----fig.wide = TRUE----------------------------------------------------------
# Retrieve the top 5 features that contribute the most to the similarity between each pair of clusters:
similarityTable5TopFeatures <- clusterFoldSimilarity(scList=singlecellObjectList, 
                                                     topNFeatures=5, 
                                                     nSubsampling=24)

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(
  head(similarityTable5TopFeatures, n=10),
  caption = 'Similarity results showing the top 5 features that most contributed to the similarity.', align = "c", 
  format = "html", table.attr = "style='width:70%;'") %>% kableExtra::kable_styling(font_size=10)

## ----fig.wide = TRUE,  out.height="400px"-------------------------------------
similarityTableAllValues <- clusterFoldSimilarity(scList=singlecellObjectList, 
                                                  sampleNames=c("human", "humanNA"), 
                                                  topN=Inf)
dim(similarityTableAllValues)

## -----------------------------------------------------------------------------
library(dplyr)
dataset1 <- "human"
dataset2 <- "humanNA"
similarityTable2 <- similarityTableAllValues %>% 
  filter(datasetL == dataset1 & datasetR == dataset2) %>% 
  arrange(desc(as.numeric(clusterL)), as.numeric(clusterR))
cls <- unique(similarityTable2$clusterL)
cls2 <- unique(similarityTable2$clusterR)
similarityMatrixAll <- t(matrix(similarityTable2$similarityValue, ncol=length(unique(similarityTable2$clusterL))))
rownames(similarityMatrixAll) <- cls
colnames(similarityMatrixAll) <- cls2

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(
  similarityMatrixAll,
  caption = 'A 2 datasets Similarity matrix.', digits = 2, align = "c", 
  format = "html", table.attr = "style='width:60%;'") %>% kableExtra::kable_styling(font_size=8)

## -----------------------------------------------------------------------------
# Mouse pancreatic single cell data
pancreasBaronMM <- scRNAseq::BaronPancreasData(which="mouse", ensembl=FALSE)

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(
  table(colData(pancreasBaronMM)$label),
  caption = 'Cell-types on pancreas dataset from Baron et al.', align = "c"
)

## -----------------------------------------------------------------------------
colData(pancreasBaronMM)$cell.type <- colData(pancreasBaronMM)$label
# Translate mouse gene ids to human ids
# *for the sake of simplicity we are going to transform to uppercase all mouse gene names
rownames(pancreasBaronMM) <- make.names(toupper(rownames(pancreasBaronMM)), unique=TRUE)
# Create seurat object
singlecell3Seurat <- CreateSeuratObject(counts=counts(pancreasBaronMM), meta.data=as.data.frame(colData(pancreasBaronMM)))

# We append the single-cell object to our list
singlecellObjectList[[3]] <- singlecell3Seurat

## ----results='hide'-----------------------------------------------------------
scObject <- singlecellObjectList[[3]]
scObject <- NormalizeData(scObject)
scObject <- FindVariableFeatures(scObject, selection.method="vst", nfeatures=2000)
scObject <- ScaleData(scObject, features=VariableFeatures(scObject))
scObject <- RunPCA(scObject, features=VariableFeatures(object=scObject))
scObject <- FindNeighbors(scObject, dims=seq(16))
scObject <- FindClusters(scObject, resolution=0.4)
singlecellObjectList[[3]] <- scObject

## ----fig.wide = TRUE----------------------------------------------------------
# We use the cell labels as a second reference, but we can also use the cluster labels if our interest is to match clusters
Idents(singlecellObjectList[[3]]) <- factor(singlecellObjectList[[3]][[]][,"cell.type"])

# We subset the most variable genes in each experiment
singlecellObjectListVariable <- lapply(singlecellObjectList, function(x){x[VariableFeatures(x),]})

# Setting the number of CPUs with BiocParallel:
BiocParallel::register(BPPARAM =  BiocParallel::MulticoreParam(workers = 6))

similarityTableHumanMouse <- clusterFoldSimilarity(scList=singlecellObjectListVariable,
                                                        sampleNames=c("human", "humanNA", "mouse"),
                                                        topN=1, 
                                                        nSubsampling=24,
                                                        parallel=TRUE)

## ----fig.wide = TRUE,  out.height="400px"-------------------------------------
similarityTableHumanMouseAll <- clusterFoldSimilarity(scList=singlecellObjectListVariable,
                                                          sampleNames=c("human", "humanNA", "mouse"),
                                                          topN=Inf,
                                                          nSubsampling=24,
                                                          parallel=TRUE)

## ----fig.wide = TRUE,  out.height="400px"-------------------------------------
ClusterFoldSimilarity::similarityHeatmap(similarityTable=similarityTableHumanMouseAll, mainDataset="humanNA")

## ----fig.wide = TRUE,  out.height="400px"-------------------------------------
# Turn-off the highlighting:
ClusterFoldSimilarity::similarityHeatmap(similarityTable=similarityTableHumanMouseAll, mainDataset="humanNA", highlightTop=FALSE)

## -----------------------------------------------------------------------------
sessionInfo()

#' Calculate cluster similarity between clusters from different single cell samples.
#'
#' `clusterFoldSimilarity()` returns a dataframe containing the best top similarities between all possible pairs of single cell samples.
#'
#' This function will calculate a similarity coeficient using the fold changes of shared features (e.g.:genes for a single-cell RNA-Seq, peaks for ATAC-Seq) among clusters, or user-defined-groups, from different samples/batches. The similarity coeficient
#' is calculated using the dotproduct of every pairwise combination of Fold Changes between a source cluster/group i from sample n and all the target clusters/groups in sample j.
#'
#' @param scList List. A list of Single Cell Experiments or Seurat objects. At least 2 are needed. The objects are expected to have cluster or label groups set as identity class.
#' @param sampleNames Character Vector. Specify the sample names, if not a number corresponding with its position on (scList).
#' @param topN Numeric. Specifies the number of target clusters with best similarity to report for each cluster comparison (default 1). If set to Inf, then all similarity values from all possible pairs of clusters are returned.
#' @param topNFeatures Numeric. Number of top features that explains the clusters similarity to report for each cluster comparison (default 1). If topN = Inf then topNFeatures is automatically set to 1.
#' @param nSubsampling Numeric. Number of random sampling of cells to achieve fold change stability (default 15).
#' @param parallel Boolean. Whether to use parallel computing using BiocParallel or not (default FALSE).
#' 
#' @return The function returns a DataFrame containing the best top similarities between all possible pairs of single cell samples. Column values are:
#' \tabular{ll}{
#'    \code{similarityValue} \tab The top similarity value calculated between datasetL:clusterL and datasetR. \cr
#'    \tab \cr
#'    \code{w} \tab Weight associated with the similarity score value. \cr
#'    \tab \cr
#'    \code{datasetL} \tab Dataset left, the dataset/sample which has been used to be compared.  \cr
#'    \tab \cr
#'    \code{clusterL} \tab Cluster left, the cluster source from datasetL which has been compared. \cr
#'    \tab \cr
#'    \code{datasetR} \tab Dataset right, the dataset/sample used for comparison against datasetL. \cr
#'    \tab \cr
#'    \code{clusterR} \tab Cluster right, the cluster target from datasetR which is being compared with the clusterL from datasetL. \cr
#'    \tab \cr
#'    \code{topFeatureConserved} \tab The features (e.g.: genes, peaks...) that most contributed to the similarity between clusterL & clusterR. \cr
#'    \tab \cr
#'    \code{featureScore} \tab The similarity score contribution for the specific topFeatureConserved (e.g.: genes, peaks...). \cr
#' }
#'
#' @examples
#' if (requireNamespace("Seurat") & requireNamespace("SeuratObject")){
#' library(ClusterFoldSimilarity)
#' library(Seurat)
#' library(SeuratObject)
#' # data dimensions
#' nfeatures <- 2000; ncells <- 400
#' # single-cell 1
#' counts <- matrix(rpois(n=nfeatures * ncells, lambda=10), nfeatures)
#' rownames(counts) <- paste0("gene",seq(nfeatures))
#' colnames(counts) <- paste0("cell",seq(ncells))
#' colData <- data.frame(cluster=sample(c("Cluster1","Cluster2","Cluster3"),size = ncells,replace = TRUE),
#'                      row.names=paste0("cell",seq(ncells)))
#' seu1 <- SeuratObject::CreateSeuratObject(counts = counts, meta.data = colData)
#' Idents(object = seu1) <- "cluster"
#' # single-cell 2
#' counts <- matrix(rpois(n=nfeatures * ncells, lambda=20), nfeatures)
#' rownames(counts) <- paste0("gene",seq(nfeatures))
#' colnames(counts) <- paste0("cell",seq(ncells))
#' colData <- data.frame(cluster=sample(c("Cluster1","Cluster2","Cluster3","Cluster4"),size = ncells,replace = TRUE),
#'                       row.names=paste0("cell",seq(ncells)))
#' seu2 <- SeuratObject::CreateSeuratObject(counts = counts, meta.data = colData)
#' Idents(object = seu2) <- "cluster"
#' # Create a list with the unprocessed single-cell datasets
#' singlecellObjectList <- list(seu1, seu2)
#' 
#' similarityTable <- clusterFoldSimilarity(scList=singlecellObjectList, sampleNames = c("sc1","sc2"))
#' head(similarityTable)
#' }
#' 
#' @author Oscar Gonzalez-Velasco
#' @import Seurat
#' @importFrom SingleCellExperiment colLabels counts
#' @importFrom stats sd
#' @importFrom utils head
#' @importFrom BiocParallel bplapply
#' @importFrom Matrix Matrix
#' @export
clusterFoldSimilarity <- function(scList=NULL, sampleNames=NULL, topN=1, topNFeatures=1, nSubsampling=15, parallel=FALSE){
  if(is.null(scList) | (length(scList)<2)){
    stop("At least two Single Cell Experiments are needed for cluster comparison.")
  }
  if(is.infinite(topN)){
    ## if topN clusters to report is set to Inf, then we return ALL similarity values from ALL the possible combination of clusters
    message("Returning similarity values from ALL pairs of clusters")
    message("  *(Note: by using topN=Inf the topNFeatures is set automatically to 1)")
    topNFeatures <- 1
  }
  isSeurat <- FALSE
  isSce <- FALSE
  ## Function starts by loading dependencies
  if(is(scList[[1]], 'Seurat')){
    if (!requireNamespace("Seurat", quietly=TRUE)) {
      stop("Package \"Seurat\" needed for this function to work. Please install it.",
           call.=FALSE)}
    isSeurat <- TRUE
    isSce <- FALSE
  }
  if(is(scList[[1]], 'SingleCellExperiment')){
    if (!requireNamespace("SingleCellExperiment", quietly=TRUE)) {
      stop("Package \"SingleCellExperiment\" needed for this function to work. Please install it.",
           call.=FALSE)}
    isSeurat <- FALSE
    isSce <- TRUE
  }
  if (!isSeurat & !isSce){
    stop("One or more objects in the input list is neither of class Seurat nor SingleDataExperiment.")
  }
  functToApply <- base::lapply
  if(isTRUE(parallel)){
    functToApply <- BiocParallel::bplapply
  }
  ## Final var names
  summaryResults <- data.frame(similarityValue=integer(),
                                w=integer(),
                                datasetL=integer(), 
                                clusterL=integer(),
                                datasetR=integer(),
                                clusterR=integer(),
                                topFeatureConserved=character(),
                                featureScore=integer(),
                                stringsAsFactors=FALSE)
  ## Select common genes in all samples 
  features <- Reduce(intersect,lapply(scList,function(x){rownames(x)}))
  if(length(features) == 0){
    stop("No common features between datasets. Please, select a subset of common variable features among all datasets.")
  }
  if(length(features) < 10){
    warning("The number of common features among datasets is: ",length(features),". More than 50 common features among all datasets is recomended.")
  }
  scList <- lapply(seq(scList), function(nScObject){
    if(isSce){
      if(is.null(SingleCellExperiment::colLabels(scList[[nScObject]]))){
        stop("No clusters specified on colLabels on sample ", nScObject)
      }
      # get groups
      if(is.factor(SingleCellExperiment::colLabels(scList[[nScObject]]))){
        groups <- droplevels(SingleCellExperiment::colLabels(scList[[nScObject]]))
      }else{
        groups <- factor(SingleCellExperiment::colLabels(scList[[nScObject]]))
      }
      clusterNames <- levels(groups)
      countData <- Matrix::Matrix(data=SingleCellExperiment::counts(scList[[nScObject]]), sparse=TRUE)
      }
    if(isSeurat){
      if(is.factor(Seurat::Idents(scList[[nScObject]]))){
        groups <- droplevels(Seurat::Idents(scList[[nScObject]]))
      }else{
        groups <- factor(Seurat::Idents(scList[[nScObject]]))
      }
      clusterNames <- levels(groups)
      countData <- Matrix::Matrix(data=Seurat::GetAssayData(scList[[nScObject]], layer="counts"), sparse=TRUE)
    }
    return(list(countData, groups, clusterNames))
  })
  clusterNames <- lapply(scList, function(scObject)scObject[[3]])
  ## Empty spaces to clear the append of message report
  spaces <- "                  "
  ## Check for dataset names given by the user:
  if(is.null(sampleNames)){
    sampleNames <- seq(length(scList))
  }else if(!(length(sampleNames) == length(scList))){
    stop("Number of names given in sampleNames does not match the length of the single-cell experiment list.")
  }
  ## Calculate cluster FoldChange pairwise values:
  markerScList <- list()
  message("Using a common set of ", length(features), " features.")
  # Testing the number of draws/subsamplings of cells needed to see *all cells*
  cellDraw <- function(n){
    percentage <- (1/3) * n
    return(sum(rep(1, n+1) / seq(1, n+1)) * (n/percentage))
  }
  #
  nOfDraws <- unlist(lapply(scList, function(scObject)lapply(c(table(scObject[[2]])), cellDraw)))
  # We select as optimal the subsampling n percentile 85% across all cell groups
  message("Using a cell subsampling of n=", nSubsampling, " (recomended n=", round(quantile(nOfDraws, probs=0.85)), ")")
  message("Computing fold changes.")
  markerScList <- functToApply(scList, function(scObject){
    pairwiseClusterFoldChange(countData=scObject[[1]], clusters=scObject[[2]], nSubsampling=nSubsampling, functToApply=functToApply)
  })
  # Remove list of single-cell object no longer needed:
  rm(scList)
  ## Main loop - samples
  intermediateResults <- lapply(seq_len(length(markerScList)), function(sampleN){
    ## Pick a sample in ascending order
    rootSampleList <- markerScList[[sampleN]]
    intermediateResults <- functToApply(seq_along(rootSampleList), function(clusterN){ # Parallel computing if selected
      ## We choose a root cluster and compare it with clusters of samples sampleN+1+...+n
      root <- rootSampleList[[clusterN]]
      for (sampleNTarget in seq(from=1, to=length(markerScList), by=1)){
        if(sampleNTarget == sampleN)next() # If root and target samples are the same, we go for the next sample and skip this loop
        sceComparative <- markerScList[[sampleNTarget]]
        results <- data.frame(similarityValue=integer(),
                              w=integer(),
                              datasetL=integer(), 
                              clusterL=integer(),
                              datasetR=integer(),
                              clusterR=integer(),
                              topFeatureConserved=character(),
                              featureScore=integer(),
                              stringsAsFactors=FALSE)
          for(clusterNTarget in seq_along(sceComparative)){
            ## The sample for comparing will be the sample_i+1
            comparative <- sceComparative[[clusterNTarget]]
            ## Comparative is in this case single cluster from sample_i+1
            textwide <- paste0("\r Comparing [cluster ", clusterNames[[sampleN]][clusterN], "] from dataset: ", sampleNames[sampleN], 
                               " with [cluster: ", clusterNames[[sampleNTarget]][clusterNTarget] , "] from dataset: ", sampleNames[sampleNTarget], spaces)
            message(textwide, appendLF=FALSE)
            ## Create a matrix A and B to compute the dot-product of all possible combinations of cluster comparison FoldChanges for each gene
            mat <- foldchangeComposition(root[features,], comparative[features,])
            ## We apply the mean, as the number of clusters between datasets could be different,
            ## just by doing the sum could be biased by the number of pair-wise FC comparisons
            matColmean <- colMeans(mat, na.rm=TRUE) # We obtain one value per gene
            topGenes <- head(colnames(mat)[order(matColmean, decreasing=TRUE)], n=topNFeatures)
            nNegative <- 0
            nPositive <- 0
            nNegative <- sum(matColmean< (-0.2), na.rm=TRUE)
            nPositive <- sum(matColmean> (0.20), na.rm=TRUE)
            ## Weight based on the number of concordant-discordant FCs:
            ## loosely based on cross-entropy
            weight <- round(log(max(abs(nPositive - nNegative), 1)), digits = 2) * sign(nPositive - nNegative) # max: Avoiding log 0
            similarity_weighted <- (sqrt(abs(sum(matColmean))) + weight) * sign(sum(matColmean) + weight)
            ## Save results
            for (gene in topGenes){
              results[nrow(results) + 1,] <- list(
                similarity_weighted, ## similarityValue
                weight, ## Weight of the score
                sampleNames[sampleN], ## datasetL
                clusterNames[[sampleN]][clusterN], ## clusterL (left; source of comparison) -> clusterN corresponding to the loop
                sampleNames[sampleNTarget], ## datasetR
                clusterNames[[sampleNTarget]][clusterNTarget], ## clusterR (right; target of comparison) -> clusterNTarget corresponding to the internal loop
                gene, ## Top feature conserved
                matColmean[gene]) ## Feature Score
            }
          }
        if(is.infinite(topN)){
        results <- do.call("rbind", by(results, results$clusterL, function(x){x[order(x[,"similarityValue"], decreasing=TRUE),]}))
        }else{
        results <- do.call("rbind", by(results, results$clusterL, function(x){head(x[order(x[,"similarityValue"], decreasing=TRUE),], n=topN*topNFeatures)}))
        }
        summaryResults <- rbind.data.frame(summaryResults,results)
      }
    return(summaryResults)
    })
  }) # End main loop - sapply
  if(is(intermediateResults[[1]], "list")){
    # We obtain a list of list containing the data.frame results
    summaryResults <- do.call("rbind", lapply(intermediateResults, function(x){do.call(rbind, x)}))
  }else if(is(intermediateResults[[1]], "data.frame")){
    # Special case in which the number of cluster is the same on all datasets, we will get a data.frame
    summaryResults <- do.call("rbind", intermediateResults)
  }
  if(is.infinite(topN)){
  message("\n Ploting heatmap using the similarity values of clusters (topN=Inf).")
  show(similarityHeatmap(similarityTable=summaryResults))
  }else{
  message("\n Ploting graph using the similarity values of clusters.")
  plotClustersGraph(similarityTable=summaryResults)
  }
  message("Returning similarity table.")
  return(summaryResults)
}

#' Plot a heatmap of the similarity values obtained using cluster fold similarity
#'
#' `similarityHeatmap()` returns a ggplot heatmap representing the similarity values between pairs of clusters as obtained from \link[ClusterFoldSimilarity]{clusterFoldSimilarity}.
#'
#' This function plots a heatmap using ggplot. It is intended to be used with the output table from \link[ClusterFoldSimilarity]{clusterFoldSimilarity}, which includes the columns: datasetL (the dataset used for comparison)
#' datasetR (the dataset against datasetL has been contrasted), clusterL (clusters from datasetL), clusterR (clusters from datasetR) and the similarityValue.
#'
#' @param similarityTable A DataFrame containing the similarities between all possible pairs of single cell samples obtained with \link[ClusterFoldSimilarity]{clusterFoldSimilarity} using the option n_top=Inf.
#' @param mainDataset Numeric. Specify the main dataset (y axis). It corresponds with the datasetL column from the similarityTable
#' @param otherDatasets Numeric. Specify some specific dataset to be ploted along the mainDataset (x axis, default: all other datasets found on datasetR column from similarity_table).
#' @param highlightTop Boolean. If the top 2 similarity values should be highlighted on the heatmap (default: TRUE)
#' 
#' @return The function returns a heatmap ggplot object. 
#' 
#' @examples
#' if (requireNamespace("Seurat") & requireNamespace("SeuratObject")){
#' library(ClusterFoldSimilarity)
#' library(Seurat)
#' library(SeuratObject)
#' # data dimensions
#' nfeatures <- 2000; ncells <- 400
#' # single-cell 1
#' counts <- matrix(rpois(n=nfeatures * ncells, lambda=10), nfeatures)
#' rownames(counts) <- paste0("gene",seq(nfeatures))
#' colnames(counts) <- paste0("cell",seq(ncells))
#' colData <- data.frame(cluster=sample(c("Cluster1","Cluster2","Cluster3"),size = ncells,replace = TRUE),
#'                      row.names=paste0("cell",seq(ncells)))
#' seu1 <- SeuratObject::CreateSeuratObject(counts = counts, meta.data = colData)
#' Idents(object = seu1) <- "cluster"
#' # single-cell 2
#' counts <- matrix(rpois(n=nfeatures * ncells, lambda=10), nfeatures)
#' rownames(counts) <- paste0("gene",seq(nfeatures))
#' colnames(counts) <- paste0("cell",seq(ncells))
#' colData <- data.frame(cluster=sample(c("Cluster1","Cluster2","Cluster3","Cluster4"),size = ncells,replace = TRUE),
#'                       row.names=paste0("cell",seq(ncells)))
#' seu2 <- SeuratObject::CreateSeuratObject(counts = counts, meta.data = colData)
#' Idents(object = seu2) <- "cluster"
#' # Create a list with the unprocessed single-cell datasets
#' singlecellObjectList <- list(seu1, seu2)
#' # Using topN = Inf by default plots a heatmap using the similarity values: 
#' similarityTableAll <- clusterFoldSimilarity(scList=singlecellObjectList, topN=Inf)
#' # Using the dataset 2 as a reference on the Y-axis of the heatmap:
#' similarityHeatmap(similarityTable=similarityTableAll, mainDataset=2, highlightTop=FALSE)
#' }
#' 
#' @author Oscar Gonzalez-Velasco
#' @importFrom dplyr %>% filter arrange
#' @importFrom ggdendro ggdendrogram
#' @import ggplot2
#' @importFrom cowplot align_plots plot_grid
#' @importFrom reshape2 melt
#' @importFrom stats hclust dist setNames
#' @export
similarityHeatmap <- function(similarityTable = NULL, 
                              mainDataset = NULL, 
                              otherDatasets = NULL, 
                              highlightTop = TRUE){
  myTable <- similarityTable
  if (is.null(mainDataset)) {
    # If no dataset is specified, we pick the first label from the datasetL values.
    mainDataset <- unique(myTable$datasetL)[1]
  }
  subMyTable <- myTable[myTable$datasetL %in% c(mainDataset),]
  if (!is.null(otherDatasets)){subMyTable <- subMyTable[subMyTable$datasetR %in% c(otherDatasets),]}
  # Order the rows by using factor levels:
  subMyTable$clusterL <- factor(subMyTable$clusterL, levels = unique(subMyTable$clusterL), ordered = TRUE)
  foundDataset <- unique(subMyTable$datasetR)
  labels_personaliz <- paste("Dataset", foundDataset)
  names(labels_personaliz) <- foundDataset
  ylabel <- paste("Dataset", mainDataset, "clusterL")
  # Create the dendrogram for Rows:
  nDistinctClasses <- length(unique(paste0(subMyTable[, "datasetR"], subMyTable[, "clusterR"])))
  subMyTable <- subMyTable[order(subMyTable$clusterR), ]
  rows <- unique(subMyTable[, "clusterL"])
  similarityMatrixAll <- matrix(subMyTable[, "similarityValue"], ncol = nDistinctClasses)
  rownames(similarityMatrixAll) <- rows
  dendoGlobalRows <- stats::hclust(stats::dist(similarityMatrixAll))
  orderingHierarchicalRows <- dendoGlobalRows$labels[dendoGlobalRows$order]
  # We split the similarity table by reference datasets:
  similarityDfList <- split(subMyTable, f = subMyTable$datasetR)
  # Create one plot per reference dataset:
  listOfPlots <- lapply(similarityDfList, function(df){
    # Contruct the Dendogram:
    dataset2 <- unique(df$datasetR)
    dataset1 <- mainDataset
    similarityMatrix <- df %>% 
      dplyr::filter(datasetL == dataset1 & datasetR == dataset2) %>% 
      dplyr::arrange(desc(clusterL), clusterR)
    
    similarityMatrix <- similarityMatrix[order(similarityMatrix$clusterR),]
    
    rows <- unique(similarityMatrix$clusterL)
    columns <- as.character(unique(similarityMatrix$clusterR))
    similarityMatrixAll <- matrix(similarityMatrix$similarityValue, ncol = length(columns))
    rownames(similarityMatrixAll) <- rows
    colnames(similarityMatrixAll) <- columns
    # Build the dendrogram
    dendo <- stats::hclust(stats::dist(t(similarityMatrixAll)))
    orderingHierarchical <- dendo$labels[dendo$order]
    dendrogram <- ggdendro::ggdendrogram(dendo, labels = FALSE) + 
      ggplot2::theme(plot.margin = ggplot2::unit(c(0, 0, 0, 0), units = "npc"), 
                     axis.text.y = ggplot2::element_blank(), 
                     axis.text.x = ggplot2::element_blank())
    df <- reshape2::melt(similarityMatrixAll)
    colnames(df) <- c("clusterL", "clusterR", "similarityValue")
    df <- df %>% dplyr::arrange(factor(clusterR, levels = orderingHierarchical))
    df$clusterR <- factor(df$clusterR, levels = orderingHierarchical)
    df$clusterL <- factor(df$clusterL, levels = orderingHierarchicalRows)
    if (isTRUE(highlightTop)) {
      highlightTop <- data.frame(clusterR=max.col(similarityMatrixAll[orderingHierarchicalRows, orderingHierarchical]), 
                                 clusterL = 1:nrow(similarityMatrixAll))
      highlightSecond <- data.frame(clusterR = apply(similarityMatrixAll[orderingHierarchicalRows, orderingHierarchical], 1, function(row) {order(row, decreasing = TRUE)[2]}), 
                                    clusterL = 1:nrow(similarityMatrixAll))
    }
    if (isFALSE(highlightTop)) {
      highlightTop <- stats::setNames(data.frame(matrix(ncol = 2, nrow = 0)), c("clusterR", "clusterL"))
      highlightSecond <- stats::setNames(data.frame(matrix(ncol = 2, nrow = 0)), c("clusterR", "clusterL"))
    }
    ggHeatmap <- ggplot2::ggplot(df) + 
      ggplot2::aes(y = clusterL, x = clusterR, fill = similarityValue) + 
      ggplot2::geom_tile() + 
      ggplot2::scale_fill_gradient2(low = "#7855ba", 
                                    mid = "#ffffff", 
                                    high = "#8a0d00", 
                                    guide = "colorbar", 
                                    name = "Similarity value") + 
      ggplot2::geom_rect(data = highlightSecond, linewidth = 0.6, fill = NA, colour = "#990B00", 
                         ggplot2::aes(xmin = clusterR - 0.5, xmax = clusterR + 0.5, ymin = clusterL - 0.5, ymax = clusterL + 0.5)) + 
      ggplot2::geom_rect(data = highlightTop, linewidth = 0.6, fill = NA, colour = "black", 
                         ggplot2::aes(xmin = clusterR - 0.5, xmax = clusterR + 0.5, ymin = clusterL - 0.5, ymax = clusterL + 0.5)) + 
      ggplot2::theme_minimal() + 
      ggplot2::coord_equal() +
      ggplot2::ylab(ylabel) + 
      ggplot2::xlab(paste("Dataset", dataset2, "clusterR")) + 
      ggplot2::theme(
        strip.text = ggplot2::element_text(face = "bold", size = ggplot2::rel(0.8)), 
        strip.background = ggplot2::element_rect(fill = "white", colour = "black", linewidth = 0.6), 
        axis.title.x = ggplot2::element_text(),
        axis.text.x = ggplot2::element_text(angle = 45, vjust = 1, hjust = 1), 
        legend.position = "left", plot.margin = ggplot2::unit(c(0, 0, 0, 0), units = "npc"))
    return(list(dendrogram, ggHeatmap))
  })
  if (length(foundDataset) > 1) {
    heatmaps <- cowplot::align_plots(plotlist = lapply(listOfPlots, function(plot){plot[[2]]}), align = "h", axis = "b")
    dendros <- cowplot::align_plots(plotlist = lapply(listOfPlots, function(plot){plot[[1]]}), align = "h", axis = "b")
    finalPlot <- cowplot::plot_grid(plotlist = c(dendros, heatmaps), align = "v", axis = "tb", rel_heights = c(0.2, 1))
  } else {
    finalPlot <- cowplot::plot_grid(listOfPlots[[1]][[1]], listOfPlots[[1]][[2]], nrow = 2, align = "v", axis = "tb", rel_heights = c(0.2, 1))
  }
  return(finalPlot)
}

#' Creates a graph plot using the similarity values calculated with ClusterFoldSimilarity().
#'
#' `plotClustersGraph()` Creates a graph plot using the similarity values calculated with ClusterFoldSimilarity().
#'
#' This function will calculate a similarity coeficient using the fold changes of shared genes among clusters of different samples/batches. The similarity coeficient
#' is calculated using the dotproduct of every pairwise combination of Fold Changes between a source cluster i of sample n and all the target clusters in sample j.
#'
#' @param similarityTable Dataframe. A table obtained from ClusterFoldSimilarity that contains the similarity values as a column "similarityValue" that represents 
#' the similarity of a source cluster to a target cluster.
#' 
#' @return This function plots a graph in which the nodes are clusters from a specific dataset, the edges represent the similarity and the direction
#' of that similarity between clusters.
#' 
#' @examples
#' if (requireNamespace("Seurat") & requireNamespace("SeuratObject")){
#' library(ClusterFoldSimilarity)
#' library(Seurat)
#' library(SeuratObject)
#' # data dimensions
#' nfeatures <- 2000; ncells <- 400
#' # single-cell 1
#' counts <- matrix(rpois(n=nfeatures * ncells, lambda=10), nfeatures)
#' rownames(counts) <- paste0("gene",seq(nfeatures))
#' colnames(counts) <- paste0("cell",seq(ncells))
#' colData <- data.frame(cluster=sample(c("Cluster1","Cluster2","Cluster3"),size = ncells,replace = TRUE),
#'                      row.names=paste0("cell",seq(ncells)))
#' seu1 <- SeuratObject::CreateSeuratObject(counts = counts, meta.data = colData)
#' Idents(object = seu1) <- "cluster"
#' # single-cell 2
#' counts <- matrix(rpois(n=nfeatures * ncells, lambda=10), nfeatures)
#' rownames(counts) <- paste0("gene",seq(nfeatures))
#' colnames(counts) <- paste0("cell",seq(ncells))
#' colData <- data.frame(cluster=sample(c("Cluster1","Cluster2","Cluster3","Cluster4"),size = ncells,replace = TRUE),
#'                       row.names=paste0("cell",seq(ncells)))
#' seu2 <- SeuratObject::CreateSeuratObject(counts = counts, meta.data = colData)
#' Idents(object = seu2) <- "cluster"
#' # Create a list with the unprocessed single-cell datasets
#' singlecellObjectList <- list(seu1, seu2)
#' 
#' similarityTable <- clusterFoldSimilarity(scList = singlecellObjectList, sampleNames = c("sc1","sc2"))
#' head(similarityTable)
#' plotClustersGraph(similarityTable=similarityTable)
#' }
#' 
#' @author Oscar Gonzalez-Velasco
#' @importFrom igraph graph_from_data_frame layout_with_fr V
#' @importFrom scales alpha
#' @importFrom graphics legend par
#' @export
plotClustersGraph <- function(similarityTable=NULL){
  ## We will use the similarity matrix to create a graph
  if(is.null(similarityTable) | !is.data.frame(similarityTable)){
    stop("A similarity table from clusterFoldSimilarity was not found.")
  }
  df <- similarityTable
  from <- paste(paste("D", df$datasetL, sep="."), paste0("C", df$clusterL), sep=".")
  to <- paste(paste("D", df$datasetR, sep="."), paste0("C", df$clusterR), sep=".")
  ## igraph
  relations <- data.frame(from=from, to=to, weight=df$similarityValue)
  g <- igraph::graph_from_data_frame(relations, directed=TRUE)
  l <- igraph::layout_with_gem(g)
  # l <- igraph::layout_with_kk(g)
  ## Retrieve the name and order of the datasets
  setNames <- unique(df$datasetL)
  cl <- as.numeric(apply(table(df$datasetL, df$clusterL)[setNames,] != 0, 1, sum))
  ## Personalized colors  
  clusterColorsPalete <- c("#FDD49E", "#D5BADB", "#7EB6D9", "#DBECDA", "#F28D35", "#4AA147", "#86608E",
                          "#3C7DA6", "#DE77AE", "#D9E8F5", "#92C791", "#D94D1A", "#F2D377")
  clCodes <- clusterColorsPalete[seq(length(setNames))]
  par(mar=c(1, 1, 3, 1));
  base::plot(g, vertex.label=igraph::V(g)$name, edge.arrow.size=.3, layout=l,
             vertex.color=scales::alpha(rep(clCodes, cl), alpha=0.8), edge.color=scales::alpha("black", alpha=0.6),
             vertex.size=15, edge.arrow.size=0.4, vertex.frame.color=NA, vertex.label.color="black", 
             vertex.label.cex=0.9, vertex.label.dist=0, edge.curved=0.2)
  legend('topleft', horiz=TRUE, y.intersp=0.5, x.intersp=0.5, legend=paste0('Dataset ', unique(df$datasetL)),
         fill=clCodes, xpd=TRUE, inset=c(0, 0), cex=0.8)
}

#' Calculate the dot product between all the possible combinations of foldchanges from diferent clusters.
#'
#' `foldchangeComposition()` returns a dataframe containing the best top similarities between all possible pairs of single cell samples.
#'
#' This function will perform the dot product of each possible combination of foldchanges, by constructing two dataframes: 
#' one with the source cluster's foldchanges and the other with the foldchange values of a target sample's cluster.
#' The computation of all the possible combinations is the hadamard product of the matrix.
#'
#' @param root Dataframe. Foldchanges between a source cluster and all the other clusters found on a sample.
#' @param comparative Dataframe. Foldchanges between a cluster and all the other clusters found on a second sample to be compared with the (root) cluster foldchanges.
#' 
#' @return A dataframe containing the hadamard product of all the possible combinations of foldchanges.
#' 
#' @keywords internal
foldchangeComposition <- function(root=NULL, comparative=NULL){
  ## Avoiding 1 column matrix to vector conversion when sub-setting index:
  root <- as.matrix(root)
  comparative <- as.matrix(comparative)
  tmp <- t(matrix(unlist(root), nrow=nrow(root), ncol=ncol(root)*ncol(comparative))) * matrix(rep(t(comparative), each=ncol(root)), ncol=nrow(root))
  colnames(tmp) <- rownames(root)
  return(tmp)
  }

#' Find cell-group communities by constructing and clustering a directed graph using the similarity values calculated by ClusterFoldSimilarity()
#'
#' `findCommunitiesSimmilarity()` Find communities by constructing and clustering graph using the similarity values calculated by ClusterFoldSimilarity().
#'
#' This function will group together nodes of the network into communities using the InfoMap community detection algorithm.
#'
#' @param similarityTable Dataframe. A table obtained from ClusterFoldSimilarity that contains the similarity values as a column "similarityValue" that represents 
#' the similarity of a source cluster to a target cluster.
#' @param nIterations Numeric. The number of iterations for the Leiden clustering algorithm (default 10).
#' @param leidenResolution Numeric. The resolution parameter for the Leiden clustering algorithm (default 1.2).
#' 
#' @return This function returns a data frame with the community that each node of the network (cell groups defined by the user) belongs to, and plots a graph in 
#' which the nodes are clusters from a specific dataset, the edges represent the similarity and the direction of that similarity between clusters.
#' \tabular{ll}{
#'    \code{sample} \tab The sample name. \cr
#'    \tab \cr
#'    \code{group} \tab The group/cluster from sample defined by the user. \cr
#'    \tab \cr
#'    \code{community} \tab Community group to which the sample:group belongs to.  \cr
#' }
#' @examples
#' if (requireNamespace("Seurat") & requireNamespace("SeuratObject")){
#' library(ClusterFoldSimilarity)
#' library(Seurat)
#' library(SeuratObject)
#' # data dimensions
#' nfeatures <- 2000; ncells <- 400
#' # single-cell 1
#' counts <- matrix(rpois(n=nfeatures * ncells, lambda=10), nfeatures)
#' rownames(counts) <- paste0("gene",seq(nfeatures))
#' colnames(counts) <- paste0("cell",seq(ncells))
#' colData <- data.frame(cluster=sample(c("Cluster1","Cluster2","Cluster3"),size = ncells,replace = TRUE),
#'                      row.names=paste0("cell",seq(ncells)))
#' seu1 <- SeuratObject::CreateSeuratObject(counts = counts, meta.data = colData)
#' Idents(object = seu1) <- "cluster"
#' # single-cell 2
#' counts <- matrix(rpois(n=nfeatures * ncells, lambda=10), nfeatures)
#' rownames(counts) <- paste0("gene",seq(nfeatures))
#' colnames(counts) <- paste0("cell",seq(ncells))
#' colData <- data.frame(cluster=sample(c("Cluster1","Cluster2","Cluster3","Cluster4"),size = ncells,replace = TRUE),
#'                       row.names=paste0("cell",seq(ncells)))
#' seu2 <- SeuratObject::CreateSeuratObject(counts = counts, meta.data = colData)
#' Idents(object = seu2) <- "cluster"
#' # Create a list with the unprocessed single-cell datasets
#' singlecellObjectList <- list(seu1, seu2)
#' 
#' similarityTable <- clusterFoldSimilarity(scList = singlecellObjectList, sampleNames = c("sc1","sc2"))
#' head(similarityTable)
#' findCommunitiesSimmilarity(similarityTable=similarityTable)
#' }
#' 
#' @author Oscar Gonzalez-Velasco
#' @importFrom igraph graph_from_data_frame layout_with_fr V cluster_infomap modularity membership
#' @importFrom scales alpha
#' @importFrom graphics legend par
#' @export
findCommunitiesSimmilarity <- function(similarityTable = NULL, nIterations = 10, leidenResolution = 1) 
{
  if (!is.data.frame(similarityTable)) {
    stop("similarityTable has to be a dataframe return by clusterFoldSimilarity")
  }
  else if (!(c("similarityValue") %in% colnames(similarityTable))) {
    stop("similarityTable has to be a dataframe return by clusterFoldSimilarity")
  }
  df <- similarityTable
  df[which(df$similarityValue < 0), "similarityValue"] <- 1e-05
  from <- paste(df$datasetL, df$clusterL, sep = "_group_")
  to <- paste(df$datasetR, df$clusterR, sep = "_group_")
  relations <- data.frame(from = from, to = to, weight = df$similarityValue)
  g <- igraph::graph_from_data_frame(relations, directed = TRUE)
  gUndirected <- igraph::as.undirected(g, mode = "collapse", edge.attr.comb = "sum")
  communities <- igraph::cluster_leiden(gUndirected, resolution_parameter = leidenResolution, n_iterations = nIterations)
  comunityMermership <- igraph::membership(communities)
  nCommunities <- length(unique(comunityMermership))
  message("Number of communities: ", nCommunities)
  l <- igraph::layout_with_gem(g)
  setNames <- unique(df$datasetL)
  cl <- as.numeric(apply(table(df$datasetL, df$clusterL)[setNames,] != 0, 1, sum))
  ## Personalized colors  
  clusterColorsPalete <- c("#FDD49E", "#D5BADB", "#7EB6D9", "#DBECDA", "#F28D35", "#4AA147", "#86608E", 
                           "#3C7DA6", "#DE77AE", "#D9E8F5", "#92C791", "#D94D1A", "#F2D377")
  clCodes <- clusterColorsPalete[seq(length(setNames))]
  # Define the colors for the communities:
  groupColores <- hcl.colors(nCommunities, palette = "Set2", alpha = 0.2, rev = FALSE, fixup = TRUE)
  groupColoresBorder <- hcl.colors(nCommunities, palette = "Set2", alpha = 0.6, rev = FALSE, fixup = TRUE)
  par(mar = c(1, 1, 3, 1))
  base::plot(communities, g, layout = l, vertex.label = gsub(pattern = "_group_", replacement = ".", x = igraph::V(g)$name), 
             vertex.color = scales::alpha(rep(clCodes,  cl), alpha = 0.8), edge.color = scales::alpha("black", alpha = 0.6), col = scales::alpha(rep(clCodes, cl), alpha = 0.8), 
             vertex.size = 15, edge.arrow.size = 0.4, vertex.frame.color = NA, 
             vertex.label.color = "black", mark.col = groupColores, 
             mark.border = groupColoresBorder, vertex.label.cex = 0.9, 
             vertex.label.dist = 0, edge.curved = 0.2)
  legend("topleft", horiz = FALSE, y.intersp = 0.8, x.intersp = 0.5, box.lwd = 0,box.col = "white",
         legend = paste0("Dataset ", unique(df$datasetL)), fill = clCodes,
         xpd = TRUE, inset = c(0, 0), cex = 0.8)
  # Return the module info as a data.frame
  samples <- unlist(lapply(strsplit(names(comunityMermership), "_group_"), function(nameEntry) nameEntry[1]))
  groups <- unlist(lapply(strsplit(names(comunityMermership), "_group_"), function(nameEntry) nameEntry[2]))
  dfMemberships <- data.frame(sample = samples, group = groups, community = c(comunityMermership))
  return(dfMemberships)
}

#' Calculate the gene mean expression Fold Change between all possible combinations of clusters.
#'
#' `pairwiseClusterFoldChange()` returns a list of dataframes containing the pairwise fold changes between all combinations of cluster.
#'
#' This function will perform fold change estimation from the mean feature´s expression between all possible combination of clusters specified on colLabels inside the sc object.
#' Bayesian Estimation of FoldChanges and Pseudocounts adapted from:
#' Florian Erhard, Estimating pseudocounts and fold changes for digital expression measurements, Bioinformatics, 
#'   Volume 34, Issue 23, December 2018, Pages 4054–4063, https://doi.org/10.1093/bioinformatics/bty471
#' Please consider citing also Erhard et. al. paper when using ClusterFoldSimilarity.
#'
#' @param countData Matrix. Normalized counts containing gene expression.
#' @param clusters Factor. A vector of corresponding cluster for each sample of (x).
#' @param nSubsampling Numeric. Number of random sampling of cells to achieve fold change stability.
#' 
#' @return A list of dataframes containing the pairwise fold changes between all combinations of cluster.
#' 
#' @author Oscar Gonzalez-Velasco
#' @importFrom methods is
#' @importFrom stats median optim pnorm quantile var
#' @importFrom BiocParallel bplapply
#' @importFrom Matrix rowMeans
#' @keywords internal
pairwiseClusterFoldChange <- function(countData, clusters, nSubsampling, functToApply){
  minNumFeatures <- 3
  cellPortion <- 1/3
  fcFunct <- function(x, y) "-"(log2(x), log2(y))
  # To avoid integer overflow use counts^2 instead of counts*counts:
  pseudoCount <- function(counts){counts + sqrt((counts^2)+1)}
  clusters <- as.factor(clusters)
  cellGroups <- split(seq_len(ncol(countData)), clusters)
  groupsByCluster <- expand.grid(levels(clusters), levels(clusters))
  groupsByCluster <- t(groupsByCluster[!(groupsByCluster[,1] == groupsByCluster[,2]),])[c(2, 1),]
  # Sub-sampling of cells - we will randomly select "nSubsampling" times the "cellPortion" of the cells and calculate the fold change mean
  listOfFolds <- functToApply(seq(1, nSubsampling), function(sampling){
    i <- lapply(cellGroups, function(cells)sample(x=cells, size=max((length(cells)*cellPortion), 1)))
    meanCounts <- vapply(i, function(i){ Matrix::rowMeans(countData[,i,drop=FALSE])}, FUN.VALUE=double(nrow(countData)))
    logFolds <- mapply(function(group1, group2){
        numerator <- meanCounts[, group1]
        denominator <- meanCounts[, group2]
        informativeFeatures <- (numerator > 0) & (denominator > 0) # Filter only genes with exprs. in BOTH conditions as informative
        if(sum(informativeFeatures) >= minNumFeatures){ # A minimum of 3 features are needed
          numerator <- numerator[informativeFeatures]
          denominator <- denominator[informativeFeatures]
          folds <- fcFunct(numerator, denominator)
          muPri <- mean(folds) # We choose mean as prior
          varPri <- max((quantile(folds, pnorm(1))-muPri)^2, (-quantile(folds, pnorm(-1))+muPri)^2) # pnorm(group1) probability in a normal distr. to find the FC group1 (mean of FC dist. = 0)
        }
      else{
        muPri <- Inf # If not enough informative features we apply the pseudocount approach
        varPri <- Inf
      }
      if (is.infinite(muPri) || is.infinite(varPri)) {
        muPri <- mean(fcFunct(pseudoCount(meanCounts[, group1]), pseudoCount(meanCounts[, group2])))
        varPri <- var(fcFunct(pseudoCount(meanCounts[, group1]), pseudoCount(meanCounts[, group2])))
      }
      sqrFunct <- function(params)((digamma(params[1])-digamma(params[2])-muPri)^2 + (trigamma(params[1])+trigamma(params[2])-varPri)^2)
      pseudoCounts <- optim(c(1, 1), sqrFunct)$par
      finalfolds <- (digamma(meanCounts[, group1]+pseudoCounts[1]) - digamma(meanCounts[, group2]+pseudoCounts[2])) / log(2)
      return(finalfolds)
      }, groupsByCluster[1,], groupsByCluster[2,])
  })
  # Calculate the element-wise mean of the sub-samplings of fold changes
  finalLogFolds <- Reduce(`+`, listOfFolds) / length(listOfFolds)
  finalLogFolds <- apply(finalLogFolds, 2,function(z)(z - stats::median(z))) # Normalization by median
  colnames(finalLogFolds) <- paste("logFC", groupsByCluster[1,], groupsByCluster[2,], sep='.')
  n <- length(levels(clusters)) - 1
  nr <- ncol(finalLogFolds)
  fcValues <- lapply(split.data.frame(t(finalLogFolds), rep(seq_len(ceiling(nr/n)), each=n, length.out=nr)), t) # A list of data.frames per cluster
  return(fcValues)
}

---
title: "ClusterFoldSimilarity: comparing cell-groups from independent single-cell experiments"
author: 
- name: Óscar González-Velasco
  affiliation: Division of Applied Bioinformatics, German Cancer Research Center DKFZ 
package: ClusterFoldSimilarity
output:
  BiocStyle::html_document:
    toc_float: true
vignette: >
  %\VignetteIndexEntry{ClusterFoldSimilarity: }
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r options, include=FALSE, echo=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(eval=TRUE, warning=FALSE, error=FALSE, message=FALSE)
```

# Installation

The package can be installed using bioconductor install manager:

```{r eval=FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("ClusterFoldSimilarity")
```

```{r setup}
library(ClusterFoldSimilarity)
```

# Introduction

Comparing single-cell data across different datasets, samples and batches has demonstrated to be challenging. `ClusterFoldSimilarity` aims to solve the complexity of comparing different single-cell datasets by computing similarity scores between clusters (or user-defined groups) from any number of independent single-cell experiments, including different species and sequencing technologies. It accomplishes this by identifying analogous fold-change patterns across cell groups that share a common set of features (such as genes). Additionally, it selects and reports the top important features that have contributed to the observed similarity, serving as a tool for feature selection.

The output is a table that contains the similarity values for all the combinations of cluster-pairs from the independent datasets. `ClusterFoldSimilarity` also includes various plotting utilities to enhance the interpretability of the similarity scores.

### Cross-species analysis and sequencing technologies (e.g.: Human vs Mouse, ATAC-Seq vs RNA-Seq)

`ClusterFoldSimilarity` is able to compare **any number** of independent experiments, including **different organisms**, making it useful for matching cell populations across different organisms, and thus, useful for inter-species analysis. Additionally, it can be used with **single-cell RNA-Seq data, single-cell ATAC-Seq data**, or more broadly, with continuous numerical data that shows changes in feature abundance across a set of common features between different groups.

### Compatibility

It can be easily integrated on any existing single-cell analysis pipeline, and it is compatible with the most used single-cell objects: `Seurat` and `SingleCellExperiment`.

Parallel computing is available through the option parallel=TRUE which make use of BiocParallel.

# Using ClusterFoldSimilarity to find similar clusters/cell-groups across datasets

Typically, `ClusterFoldSimilarity` will receive as input either a list of two or more `Seurat` or `SingleCellExperiment` objects.

`ClusterFoldSimilarity` will obtain the **raw count data** from these objects ( `GetAssayData(assay, layer = "counts")` in the case of `Seurat`, or `counts()` for `SingleCellExperiment` object), and **group or cluster label information** (using `Idents()` function from `Seurat`, or `colLabels()` for `SingleCellExperiment` ).

For the sake of illustration, we will employ the scRNAseq package, which contains numerous individual-cell datasets ready for download and encompassing samples from both human and mouse origins. In this example, we specifically utilize 2 human single-cell datasets obtained from the pancreas.

```{r construct }
library(Seurat)
library(scRNAseq)
library(dplyr)
# Human pancreatic single cell data 1
pancreasMuraro <- scRNAseq::MuraroPancreasData(ensembl=FALSE)
pancreasMuraro <- pancreasMuraro[,rownames(colData(pancreasMuraro)[!is.na(colData(pancreasMuraro)$label),])]
colData(pancreasMuraro)$cell.type <- colData(pancreasMuraro)$label
rownames(pancreasMuraro) <- make.names(unlist(lapply(strsplit(rownames(pancreasMuraro), split="__"), function(x)x[[1]])), unique = TRUE)
singlecell1Seurat <- CreateSeuratObject(counts=counts(pancreasMuraro), meta.data=as.data.frame(colData(pancreasMuraro)))
```

```{r ,  echo=FALSE}
knitr::kable(
  table(colData(pancreasMuraro)$label),
  caption = 'Cell-types on pancreas dataset from Muraro et al.', align = "c"
)
```

```{r }
# Human pancreatic single cell data 2
pancreasBaron <- scRNAseq::BaronPancreasData(which="human", ensembl=FALSE)
colData(pancreasBaron)$cell.type <- colData(pancreasBaron)$label
rownames(pancreasBaron) <- make.names(rownames(pancreasBaron), unique = TRUE)

singlecell2Seurat <- CreateSeuratObject(counts=counts(pancreasBaron), meta.data=as.data.frame(colData(pancreasBaron)))
```

```{r ,  echo=FALSE}
knitr::kable(
  table(colData(pancreasBaron)$label),
  caption = 'Cell-types on pancreas dataset from Baron et al.', align = "c"
)
```

As we want to perform clustering analysis for later comparison of these cluster groups using `ClusterFoldSimilarity`, we first need to normalize and identify variable features for each dataset independently.

*Note: these steps should be done tailored to each independent dataset, here we apply the same parameters for the sake of simplicity:*

```{r,  results='hide'}
# Create a list with the unprocessed single-cell datasets
singlecellObjectList <- list(singlecell1Seurat, singlecell2Seurat)
# Apply the same processing to each dataset and return a list of single-cell analysis
singlecellObjectList <- lapply(X=singlecellObjectList, FUN=function(scObject){
scObject <- NormalizeData(scObject)
scObject <- FindVariableFeatures(scObject, selection.method="vst", nfeatures=2000)
scObject <- ScaleData(scObject, features=VariableFeatures(scObject))
scObject <- RunPCA(scObject, features=VariableFeatures(object=scObject))
scObject <- FindNeighbors(scObject, dims=seq(16))
scObject <- FindClusters(scObject, resolution=0.4)
})
```

Once we have all of our single-cell datasets analyzed independently, we can compute the similarity values. `clusterFoldSimilarity()` takes as arguments:

-   `scList`: a list of single-cell objects (mandatory) either of class `Seurat` or of class `SingleCellExperiment`.
-   `sampleNames`: vector with names for each of the datasets. If not set the datasets will be named in the given order as: *1, 2, ..., N*.
-   `topN`: the top n most similar clusters/groups to report for each cluster/group (default: `1`, the top most similar cluster). If set to `Inf` it will return the values from all the possible cluster-pairs.
-   `topNFeatures`: the top *n* features (e.g.: genes) that contribute to the observed similarity between the pair of clusters (default: `1`, the top contributing gene). If a negative number, the tool will report the *n* most dissimilar features.
-   `nSubsampling`: number of subsamplings (1/3 of cells on each iteration) at group level for calculating the fold-changes (default: `15`). At start, the tool will report a message with the recommended number of subsamplings for the given data (average n of subsamplings needed to observe all cells).
-   `parallel`: whether to use parallel computing with multiple threads or not (default: `FALSE`). If we want to use a specific single-cell experiment for annotation (from which we know a ground-truth label, e.g. cell type, cell cycle, treatment... etc.), we can use that label to directly compare the single-cell datasets.

Here we will use the annotated pancreas cell-type labels from the dataset 1 to illustrate how to match clusters to cell-types using a reference dataset:

```{r , fig.wide = TRUE}
# Assign cell-type annotated from the original study to the cell labels:
Idents(singlecellObjectList[[1]]) <- factor(singlecellObjectList[[1]][[]][, "cell.type"])

library(ClusterFoldSimilarity)
similarityTable <- clusterFoldSimilarity(scList=singlecellObjectList, 
                                         sampleNames=c("human", "humanNA"),
                                         topN=1, 
                                         nSubsampling=24)

```

```{r ,  echo=FALSE}
knitr::kable(
  head(similarityTable), caption = 'A table of the first 10 rows of the similarity results.', align = "c", 
  format = "html", table.attr = "style='width:70%;'") %>% kableExtra::kable_styling(font_size=10)
```

A data.frame with the results is returned containing:

-   `similarityValue`: The top similarity value calculated between datasetL:clusterL and datasetR.
-   `w`: Weight associated with the similarity score value.
-   `datasetL`: Dataset left, the dataset/sample which has been used to be compared. 
-   `clusterL`: Cluster left, the cluster source from datasetL which has been compared.
-   `datasetR`: Dataset right, the dataset/sample used for comparison against datasetL.
-   `clusterR`: Cluster right, the cluster target from datasetR which is being compared with the clusterL from datasetL.
-   `topFeatureConserved`: The features (e.g.: genes, peaks...) that most contributed to the similarity between clusterL & clusterR.
-   `featureScore`: The similarity score contribution for the specific topFeatureConserved (e.g.: genes, peaks...).

By default, `clusterFoldSimilarity()` will plot a graph network that visualizes the connections between the clusters from the different datasets using the similarity table that has been obtained. The arrows point in the direction of the similarity (datasetL:clusterL -\> datasetR:clusterR); it can be useful for identifying relationships between groups of clusters and cell-populations that tend to be more similar. The graph plot can also be obtained by using the function `plotClustersGraph()` from this package, using as input the similarity table.

In this example, as we have information regarding cell-type labels, we can check how the cell types match by calculating the most abundant cell type on each of the similar clusters:

```{r}
typeCount <- singlecellObjectList[[2]][[]] %>% 
  group_by(seurat_clusters) %>% 
  count(cell.type) %>%
  arrange(desc(n), .by_group = TRUE) %>% 
  filter(row_number()==1)
```

```{r ,  echo=FALSE}
knitr::kable(
  cbind.data.frame(typeCount, 
                 matched.type=rep(table(typeCount$seurat_clusters), x=similarityTable[similarityTable$datasetL == "humanNA",]$clusterR)),
  caption = 'Cell-type label matching on pancreas single-cell data.', align = "c"
)
```

## Analyzing graph communities to identify super-groups of similar cell populations

To easily analyze and identify the similarities between the different datasets and cell-groups, we can find the communities that constitute the directed graph (cluster the nodes based on the graph´s closely-related elements).

We can make so by using the function `findCommunitiesSimmilarity()` from the `ClusterFoldSimilarity` package. It uses the InfoMap algorithm to find the best fitting communities. We just need the similarity table obtained from `clusterFoldSimilarity()` as explained on the previous section, the function will plot the graph with the communities and return a data frame containing the community that each sample & cluster/group belongs to.

```{r, fig.wide = TRUE}
cellCommunities <- findCommunitiesSimmilarity(similarityTable = similarityTable)
```

```{r,  echo=FALSE}
knitr::kable(
  head(cellCommunities)
)
```

## Retrieving the top-n similarities

If we suspect that clusters could be related with more than one cluster of other datasets, we can retrieve the top n similarities for each cluster:

```{r, fig.wide = TRUE}
# Retrieve the top 3 similar cluster for each of the clusters:
similarityTable3Top <- clusterFoldSimilarity(scList=singlecellObjectList, 
                                             topN=3,
                                             sampleNames=c("human", "humanNA"), 
                                             nSubsampling=24)
```

```{r , echo=FALSE}
knitr::kable(
  head(similarityTable3Top),
  caption = 'Similarity results showing the top 3 most similar clusters.', align = "c", 
  format = "html", table.attr = "style='width:70%;'") %>% kableExtra::kable_styling(font_size=10)
```

## Obtaining the top-n feature markers

If we are interested on the features that contribute the most to the similarity, we can retrieve the top n features:

```{r, fig.wide = TRUE}
# Retrieve the top 5 features that contribute the most to the similarity between each pair of clusters:
similarityTable5TopFeatures <- clusterFoldSimilarity(scList=singlecellObjectList, 
                                                     topNFeatures=5, 
                                                     nSubsampling=24)
```

```{r ,  echo=FALSE}
knitr::kable(
  head(similarityTable5TopFeatures, n=10),
  caption = 'Similarity results showing the top 5 features that most contributed to the similarity.', align = "c", 
  format = "html", table.attr = "style='width:70%;'") %>% kableExtra::kable_styling(font_size=10)
```

## Retrieving all the similarity values and plotting a similarity heatmap

Sometimes it is useful to retrieve all the similarity values for downstream analysis (e.g. identify more than one cluster that is similar to a cluster of interest, finding the most dissimilar clusters, etc). To obtain all the values, we need to specify `topN=Inf`.

By default, `clusterFoldSimilarity` creates a heatmap plot with the computed similarity values (from the perspective of the first dataset found on `scList`; to modify this plot see the following section). The top 2 similarities for each group within dataset 1 (heatmap row-wise) are highlighted with colored borders.

```{r , fig.wide = TRUE,  out.height="400px" }
similarityTableAllValues <- clusterFoldSimilarity(scList=singlecellObjectList, 
                                                  sampleNames=c("human", "humanNA"), 
                                                  topN=Inf)
dim(similarityTableAllValues)
```

For downstream analysis of the similarities, it can be convenient to create a matrix with all the scores from the comparison of two datasets:

```{r}
library(dplyr)
dataset1 <- "human"
dataset2 <- "humanNA"
similarityTable2 <- similarityTableAllValues %>% 
  filter(datasetL == dataset1 & datasetR == dataset2) %>% 
  arrange(desc(as.numeric(clusterL)), as.numeric(clusterR))
cls <- unique(similarityTable2$clusterL)
cls2 <- unique(similarityTable2$clusterR)
similarityMatrixAll <- t(matrix(similarityTable2$similarityValue, ncol=length(unique(similarityTable2$clusterL))))
rownames(similarityMatrixAll) <- cls
colnames(similarityMatrixAll) <- cls2
```

```{r, echo=FALSE}
knitr::kable(
  similarityMatrixAll,
  caption = 'A 2 datasets Similarity matrix.', digits = 2, align = "c", 
  format = "html", table.attr = "style='width:60%;'") %>% kableExtra::kable_styling(font_size=8)
```

# Using ClusterFoldSimilarity across species and numerous datasets:

`ClusterFoldSimilarity` can compare **any number** of independent studies, including **different organisms**, making it useful for inter-species analysis. Also, it can be used on different sequencing data technologies: e.g.: compare single-cell **ATAC-Seq VS RNA-seq**.

In this example, we are going to add a pancreas single-cell dataset from **Mouse** to the 2 existing ones from **Human** that we have processed in the previous steps.

```{r}
# Mouse pancreatic single cell data
pancreasBaronMM <- scRNAseq::BaronPancreasData(which="mouse", ensembl=FALSE)
```

```{r ,  echo=FALSE}
knitr::kable(
  table(colData(pancreasBaronMM)$label),
  caption = 'Cell-types on pancreas dataset from Baron et al.', align = "c"
)
```

```{r }
colData(pancreasBaronMM)$cell.type <- colData(pancreasBaronMM)$label
# Translate mouse gene ids to human ids
# *for the sake of simplicity we are going to transform to uppercase all mouse gene names
rownames(pancreasBaronMM) <- make.names(toupper(rownames(pancreasBaronMM)), unique=TRUE)
# Create seurat object
singlecell3Seurat <- CreateSeuratObject(counts=counts(pancreasBaronMM), meta.data=as.data.frame(colData(pancreasBaronMM)))

# We append the single-cell object to our list
singlecellObjectList[[3]] <- singlecell3Seurat
```

Now, we process the new single-cell dataset from mouse, and we calculate the similarity scores between the 3 independent datasets.

```{r,  results='hide'}
scObject <- singlecellObjectList[[3]]
scObject <- NormalizeData(scObject)
scObject <- FindVariableFeatures(scObject, selection.method="vst", nfeatures=2000)
scObject <- ScaleData(scObject, features=VariableFeatures(scObject))
scObject <- RunPCA(scObject, features=VariableFeatures(object=scObject))
scObject <- FindNeighbors(scObject, dims=seq(16))
scObject <- FindClusters(scObject, resolution=0.4)
singlecellObjectList[[3]] <- scObject
```

This time we will make use of the option parallel=TRUE. We can set the specific number of CPUs to use using `BiocParallel::register()` 

```{r, fig.wide = TRUE}
# We use the cell labels as a second reference, but we can also use the cluster labels if our interest is to match clusters
Idents(singlecellObjectList[[3]]) <- factor(singlecellObjectList[[3]][[]][,"cell.type"])

# We subset the most variable genes in each experiment
singlecellObjectListVariable <- lapply(singlecellObjectList, function(x){x[VariableFeatures(x),]})

# Setting the number of CPUs with BiocParallel:
BiocParallel::register(BPPARAM =  BiocParallel::MulticoreParam(workers = 6))

similarityTableHumanMouse <- clusterFoldSimilarity(scList=singlecellObjectListVariable,
                                                        sampleNames=c("human", "humanNA", "mouse"),
                                                        topN=1, 
                                                        nSubsampling=24,
                                                        parallel=TRUE)
```

We can compute and visualize with a heatmap all the similarities for each cluster/group of cells from the 3 datasets using `topN=Inf`. Additionally, we can use the function `similarityHeatmap()` from this package to plot the heatmap with the datasets in a different order, or just plot the 2 datasets we are interested in. The top 2 similarities are highlighted to help visualizing the best matching groups.

```{r , fig.wide = TRUE,  out.height="400px" }
similarityTableHumanMouseAll <- clusterFoldSimilarity(scList=singlecellObjectListVariable,
                                                          sampleNames=c("human", "humanNA", "mouse"),
                                                          topN=Inf,
                                                          nSubsampling=24,
                                                          parallel=TRUE)
```

As the similarity values might not be symmetric (e.g. a cluster A from D1 showing the top similarity to B from D2, might not be the top similar cluster to B from D2), we can select which dataset to plot in the Y-axis:

```{r , fig.wide = TRUE,  out.height="400px" }
ClusterFoldSimilarity::similarityHeatmap(similarityTable=similarityTableHumanMouseAll, mainDataset="humanNA")
```

Additionally, we can turn-off the highlight using `highlightTop=FALSE`

```{r , fig.wide = TRUE,  out.height="400px" }
# Turn-off the highlighting:
ClusterFoldSimilarity::similarityHeatmap(similarityTable=similarityTableHumanMouseAll, mainDataset="humanNA", highlightTop=FALSE)
```

# Similarity score calculation

`ClusterFoldSimilarity` does not need to integrate the data, or apply any batch correction techniques across the datasets that we aim to analyze, which makes it less prone to data-loss or noise. The similarity value is based on the fold-changes between clusters/groups of cells defined by the user. These fold-changes from different independent datasets are first computed using a Bayesian approach, we calculate this fold-change distribution using a permutation analysis that shrink the fold-changes with no biological meaning. These differences in abundance are then combined using a pairwise dot product approach, after adding these feature contributions and applying a fold-change concordance weight, a similarity value is obtained for each of the clusters of each of the datasets present.

# Session information {.unnumbered}

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_clusters_graph.r
\name{plotClustersGraph}
\alias{plotClustersGraph}
\title{Creates a graph plot using the similarity values calculated with ClusterFoldSimilarity().}
\usage{
plotClustersGraph(similarityTable = NULL)
}
\arguments{
\item{similarityTable}{Dataframe. A table obtained from ClusterFoldSimilarity that contains the similarity values as a column "similarityValue" that represents 
the similarity of a source cluster to a target cluster.}
}
\value{
This function plots a graph in which the nodes are clusters from a specific dataset, the edges represent the similarity and the direction
of that similarity between clusters.
}
\description{
`plotClustersGraph()` Creates a graph plot using the similarity values calculated with ClusterFoldSimilarity().
}
\details{
This function will calculate a similarity coeficient using the fold changes of shared genes among clusters of different samples/batches. The similarity coeficient
is calculated using the dotproduct of every pairwise combination of Fold Changes between a source cluster i of sample n and all the target clusters in sample j.
}
\examples{
if (requireNamespace("Seurat") & requireNamespace("SeuratObject")){
library(ClusterFoldSimilarity)
library(Seurat)
library(SeuratObject)
# data dimensions
nfeatures <- 2000; ncells <- 400
# single-cell 1
counts <- matrix(rpois(n=nfeatures * ncells, lambda=10), nfeatures)
rownames(counts) <- paste0("gene",seq(nfeatures))
colnames(counts) <- paste0("cell",seq(ncells))
colData <- data.frame(cluster=sample(c("Cluster1","Cluster2","Cluster3"),size = ncells,replace = TRUE),
                     row.names=paste0("cell",seq(ncells)))
seu1 <- SeuratObject::CreateSeuratObject(counts = counts, meta.data = colData)
Idents(object = seu1) <- "cluster"
# single-cell 2
counts <- matrix(rpois(n=nfeatures * ncells, lambda=10), nfeatures)
rownames(counts) <- paste0("gene",seq(nfeatures))
colnames(counts) <- paste0("cell",seq(ncells))
colData <- data.frame(cluster=sample(c("Cluster1","Cluster2","Cluster3","Cluster4"),size = ncells,replace = TRUE),
                      row.names=paste0("cell",seq(ncells)))
seu2 <- SeuratObject::CreateSeuratObject(counts = counts, meta.data = colData)
Idents(object = seu2) <- "cluster"
# Create a list with the unprocessed single-cell datasets
singlecellObjectList <- list(seu1, seu2)

similarityTable <- clusterFoldSimilarity(scList = singlecellObjectList, sampleNames = c("sc1","sc2"))
head(similarityTable)
plotClustersGraph(similarityTable=similarityTable)
}

}
\author{
Oscar Gonzalez-Velasco
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/similarity_heatmap.r
\name{similarityHeatmap}
\alias{similarityHeatmap}
\title{Plot a heatmap of the similarity values obtained using cluster fold similarity}
\usage{
similarityHeatmap(
  similarityTable = NULL,
  mainDataset = NULL,
  otherDatasets = NULL,
  highlightTop = TRUE
)
}
\arguments{
\item{similarityTable}{A DataFrame containing the similarities between all possible pairs of single cell samples obtained with \link[ClusterFoldSimilarity]{clusterFoldSimilarity} using the option n_top=Inf.}

\item{mainDataset}{Numeric. Specify the main dataset (y axis). It corresponds with the datasetL column from the similarityTable}

\item{otherDatasets}{Numeric. Specify some specific dataset to be ploted along the mainDataset (x axis, default: all other datasets found on datasetR column from similarity_table).}

\item{highlightTop}{Boolean. If the top 2 similarity values should be highlighted on the heatmap (default: TRUE)}
}
\value{
The function returns a heatmap ggplot object.
}
\description{
`similarityHeatmap()` returns a ggplot heatmap representing the similarity values between pairs of clusters as obtained from \link[ClusterFoldSimilarity]{clusterFoldSimilarity}.
}
\details{
This function plots a heatmap using ggplot. It is intended to be used with the output table from \link[ClusterFoldSimilarity]{clusterFoldSimilarity}, which includes the columns: datasetL (the dataset used for comparison)
datasetR (the dataset against datasetL has been contrasted), clusterL (clusters from datasetL), clusterR (clusters from datasetR) and the similarityValue.
}
\examples{
if (requireNamespace("Seurat") & requireNamespace("SeuratObject")){
library(ClusterFoldSimilarity)
library(Seurat)
library(SeuratObject)
# data dimensions
nfeatures <- 2000; ncells <- 400
# single-cell 1
counts <- matrix(rpois(n=nfeatures * ncells, lambda=10), nfeatures)
rownames(counts) <- paste0("gene",seq(nfeatures))
colnames(counts) <- paste0("cell",seq(ncells))
colData <- data.frame(cluster=sample(c("Cluster1","Cluster2","Cluster3"),size = ncells,replace = TRUE),
                     row.names=paste0("cell",seq(ncells)))
seu1 <- SeuratObject::CreateSeuratObject(counts = counts, meta.data = colData)
Idents(object = seu1) <- "cluster"
# single-cell 2
counts <- matrix(rpois(n=nfeatures * ncells, lambda=10), nfeatures)
rownames(counts) <- paste0("gene",seq(nfeatures))
colnames(counts) <- paste0("cell",seq(ncells))
colData <- data.frame(cluster=sample(c("Cluster1","Cluster2","Cluster3","Cluster4"),size = ncells,replace = TRUE),
                      row.names=paste0("cell",seq(ncells)))
seu2 <- SeuratObject::CreateSeuratObject(counts = counts, meta.data = colData)
Idents(object = seu2) <- "cluster"
# Create a list with the unprocessed single-cell datasets
singlecellObjectList <- list(seu1, seu2)
# Using topN = Inf by default plots a heatmap using the similarity values: 
similarityTableAll <- clusterFoldSimilarity(scList=singlecellObjectList, topN=Inf)
# Using the dataset 2 as a reference on the Y-axis of the heatmap:
similarityHeatmap(similarityTable=similarityTableAll, mainDataset=2, highlightTop=FALSE)
}

}
\author{
Oscar Gonzalez-Velasco
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cluster_fold_similarity.r
\name{clusterFoldSimilarity}
\alias{clusterFoldSimilarity}
\title{Calculate cluster similarity between clusters from different single cell samples.}
\usage{
clusterFoldSimilarity(
  scList = NULL,
  sampleNames = NULL,
  topN = 1,
  topNFeatures = 1,
  nSubsampling = 15,
  parallel = FALSE
)
}
\arguments{
\item{scList}{List. A list of Single Cell Experiments or Seurat objects. At least 2 are needed. The objects are expected to have cluster or label groups set as identity class.}

\item{sampleNames}{Character Vector. Specify the sample names, if not a number corresponding with its position on (scList).}

\item{topN}{Numeric. Specifies the number of target clusters with best similarity to report for each cluster comparison (default 1). If set to Inf, then all similarity values from all possible pairs of clusters are returned.}

\item{topNFeatures}{Numeric. Number of top features that explains the clusters similarity to report for each cluster comparison (default 1). If topN = Inf then topNFeatures is automatically set to 1.}

\item{nSubsampling}{Numeric. Number of random sampling of cells to achieve fold change stability (default 15).}

\item{parallel}{Boolean. Whether to use parallel computing using BiocParallel or not (default FALSE).}
}
\value{
The function returns a DataFrame containing the best top similarities between all possible pairs of single cell samples. Column values are:
\tabular{ll}{
   \code{similarityValue} \tab The top similarity value calculated between datasetL:clusterL and datasetR. \cr
   \tab \cr
   \code{w} \tab Weight associated with the similarity score value. \cr
   \tab \cr
   \code{datasetL} \tab Dataset left, the dataset/sample which has been used to be compared.  \cr
   \tab \cr
   \code{clusterL} \tab Cluster left, the cluster source from datasetL which has been compared. \cr
   \tab \cr
   \code{datasetR} \tab Dataset right, the dataset/sample used for comparison against datasetL. \cr
   \tab \cr
   \code{clusterR} \tab Cluster right, the cluster target from datasetR which is being compared with the clusterL from datasetL. \cr
   \tab \cr
   \code{topFeatureConserved} \tab The features (e.g.: genes, peaks...) that most contributed to the similarity between clusterL & clusterR. \cr
   \tab \cr
   \code{featureScore} \tab The similarity score contribution for the specific topFeatureConserved (e.g.: genes, peaks...). \cr
}
}
\description{
`clusterFoldSimilarity()` returns a dataframe containing the best top similarities between all possible pairs of single cell samples.
}
\details{
This function will calculate a similarity coeficient using the fold changes of shared features (e.g.:genes for a single-cell RNA-Seq, peaks for ATAC-Seq) among clusters, or user-defined-groups, from different samples/batches. The similarity coeficient
is calculated using the dotproduct of every pairwise combination of Fold Changes between a source cluster/group i from sample n and all the target clusters/groups in sample j.
}
\examples{
if (requireNamespace("Seurat") & requireNamespace("SeuratObject")){
library(ClusterFoldSimilarity)
library(Seurat)
library(SeuratObject)
# data dimensions
nfeatures <- 2000; ncells <- 400
# single-cell 1
counts <- matrix(rpois(n=nfeatures * ncells, lambda=10), nfeatures)
rownames(counts) <- paste0("gene",seq(nfeatures))
colnames(counts) <- paste0("cell",seq(ncells))
colData <- data.frame(cluster=sample(c("Cluster1","Cluster2","Cluster3"),size = ncells,replace = TRUE),
                     row.names=paste0("cell",seq(ncells)))
seu1 <- SeuratObject::CreateSeuratObject(counts = counts, meta.data = colData)
Idents(object = seu1) <- "cluster"
# single-cell 2
counts <- matrix(rpois(n=nfeatures * ncells, lambda=20), nfeatures)
rownames(counts) <- paste0("gene",seq(nfeatures))
colnames(counts) <- paste0("cell",seq(ncells))
colData <- data.frame(cluster=sample(c("Cluster1","Cluster2","Cluster3","Cluster4"),size = ncells,replace = TRUE),
                      row.names=paste0("cell",seq(ncells)))
seu2 <- SeuratObject::CreateSeuratObject(counts = counts, meta.data = colData)
Idents(object = seu2) <- "cluster"
# Create a list with the unprocessed single-cell datasets
singlecellObjectList <- list(seu1, seu2)

similarityTable <- clusterFoldSimilarity(scList=singlecellObjectList, sampleNames = c("sc1","sc2"))
head(similarityTable)
}

}
\author{
Oscar Gonzalez-Velasco
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_communities_simmilarity.R
\name{findCommunitiesSimmilarity}
\alias{findCommunitiesSimmilarity}
\title{Find cell-group communities by constructing and clustering a directed graph using the similarity values calculated by ClusterFoldSimilarity()}
\usage{
findCommunitiesSimmilarity(similarityTable = NULL)
}
\arguments{
\item{similarityTable}{Dataframe. A table obtained from ClusterFoldSimilarity that contains the similarity values as a column "similarityValue" that represents 
the similarity of a source cluster to a target cluster.}
}
\value{
This function returns a data frame with the community that each node of the network (cell groups defined by the user) belongs to, and plots a graph in 
which the nodes are clusters from a specific dataset, the edges represent the similarity and the direction of that similarity between clusters.
\tabular{ll}{
   \code{sample} \tab The sample name. \cr
   \tab \cr
   \code{group} \tab The group/cluster from sample defined by the user. \cr
   \tab \cr
   \code{community} \tab Community group to which the sample:group belongs to.  \cr
}
}
\description{
`findCommunitiesSimmilarity()` Find communities by constructing and clustering graph using the similarity values calculated by ClusterFoldSimilarity().
}
\details{
This function will group together nodes of the network into communities using the InfoMap community detection algorithm.
}
\examples{
if (requireNamespace("Seurat") & requireNamespace("SeuratObject")){
library(ClusterFoldSimilarity)
library(Seurat)
library(SeuratObject)
# data dimensions
nfeatures <- 2000; ncells <- 400
# single-cell 1
counts <- matrix(rpois(n=nfeatures * ncells, lambda=10), nfeatures)
rownames(counts) <- paste0("gene",seq(nfeatures))
colnames(counts) <- paste0("cell",seq(ncells))
colData <- data.frame(cluster=sample(c("Cluster1","Cluster2","Cluster3"),size = ncells,replace = TRUE),
                     row.names=paste0("cell",seq(ncells)))
seu1 <- SeuratObject::CreateSeuratObject(counts = counts, meta.data = colData)
Idents(object = seu1) <- "cluster"
# single-cell 2
counts <- matrix(rpois(n=nfeatures * ncells, lambda=10), nfeatures)
rownames(counts) <- paste0("gene",seq(nfeatures))
colnames(counts) <- paste0("cell",seq(ncells))
colData <- data.frame(cluster=sample(c("Cluster1","Cluster2","Cluster3","Cluster4"),size = ncells,replace = TRUE),
                      row.names=paste0("cell",seq(ncells)))
seu2 <- SeuratObject::CreateSeuratObject(counts = counts, meta.data = colData)
Idents(object = seu2) <- "cluster"
# Create a list with the unprocessed single-cell datasets
singlecellObjectList <- list(seu1, seu2)

similarityTable <- clusterFoldSimilarity(scList = singlecellObjectList, sampleNames = c("sc1","sc2"))
head(similarityTable)
findCommunitiesSimmilarity(similarityTable=similarityTable)
}

}
\author{
Oscar Gonzalez-Velasco
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/foldchange_composition.r
\name{foldchangeComposition}
\alias{foldchangeComposition}
\title{Calculate the dot product between all the possible combinations of foldchanges from diferent clusters.}
\usage{
foldchangeComposition(root = NULL, comparative = NULL)
}
\arguments{
\item{root}{Dataframe. Foldchanges between a source cluster and all the other clusters found on a sample.}

\item{comparative}{Dataframe. Foldchanges between a cluster and all the other clusters found on a second sample to be compared with the (root) cluster foldchanges.}
}
\value{
A dataframe containing the hadamard product of all the possible combinations of foldchanges.
}
\description{
`foldchangeComposition()` returns a dataframe containing the best top similarities between all possible pairs of single cell samples.
}
\details{
This function will perform the dot product of each possible combination of foldchanges, by constructing two dataframes: 
one with the source cluster's foldchanges and the other with the foldchange values of a target sample's cluster.
The computation of all the possible combinations is the hadamard product of the matrix.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pairwise_cluster_fold_change.r
\name{pairwiseClusterFoldChange}
\alias{pairwiseClusterFoldChange}
\title{Calculate the gene mean expression Fold Change between all possible combinations of clusters.}
\usage{
pairwiseClusterFoldChange(countData, clusters, nSubsampling, functToApply)
}
\arguments{
\item{countData}{Matrix. Normalized counts containing gene expression.}

\item{clusters}{Factor. A vector of corresponding cluster for each sample of (x).}

\item{nSubsampling}{Numeric. Number of random sampling of cells to achieve fold change stability.}
}
\value{
A list of dataframes containing the pairwise fold changes between all combinations of cluster.
}
\description{
`pairwiseClusterFoldChange()` returns a list of dataframes containing the pairwise fold changes between all combinations of cluster.
}
\details{
This function will perform fold change estimation from the mean feature´s expression between all possible combination of clusters specified on colLabels inside the sc object.
Bayesian Estimation of FoldChanges and Pseudocounts adapted from:
Florian Erhard, Estimating pseudocounts and fold changes for digital expression measurements, Bioinformatics, 
  Volume 34, Issue 23, December 2018, Pages 4054–4063, https://doi.org/10.1093/bioinformatics/bty471
Please consider citing also Erhard et. al. paper when using ClusterFoldSimilarity.
}
\author{
Oscar Gonzalez-Velasco
}
\keyword{internal}
